""" serviceconf: This module contains the Gui() class which contains the methods pertaining to the gui only """
# serviceconf.py
# Copyright (C) 2001 Red Hat, Inc.
# Authors: Tim Powers <timp@redhat.com>
#          Bill Nottingham <notting@redhat.com>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

import sys
import gtk
import GTK, GDK
import gnome
import gnome.ui
import libglade
import signal
import os
import string
import gnome.help
import CheckList
import servicemethods
from translate import _, N_, cat

VERSION = "0.6.6"

class Gui:
    """This class handles everything gui for the serviceconf application"""
    def __init__(self):

        signal.signal (signal.SIGINT, signal.SIG_DFL)

        self.ServiceMethods = servicemethods.ServiceMethods()
        
        self.xml = libglade.GladeXML("/usr/share/serviceconf/serviceconf.glade", domain="serviceconf")

        # map the event signals to specific methods
        self.xml.signal_autoconnect(
            { "on_mainwin_delete_event" : gtk.mainquit,
              "on_mainwin_hide" : gtk.mainquit,
              "on_mnuRescan_activate" : self.on_mnuRescan_activate,
              "on_mnuSave_activate" : self.on_btnSave_clicked,
              "on_mnuRevert_activate" : self.on_btnRevert_clicked,
              "on_mnuExit_activate" : gtk.mainquit,
              "on_mnuStart_activate" : self.on_btnStart_clicked,
              "on_mnuStop_activate" : self.on_btnStop_clicked,
              "on_mnuRestart_activate" : self.on_btnRestart_clicked,
              "on_pmnStart_activate" : self.on_btnStart_clicked,
              "on_pmnStop_activate" : self.on_btnStop_clicked,
              "on_pmnRestart_activate" : self.on_btnRestart_clicked,
              "on_btnSave_clicked" : self.on_btnSave_clicked,
              "on_btnRevert_clicked" : self.on_btnRevert_clicked,
              "on_btnExit_clicked" : gtk.mainquit,
              "on_btnStart_clicked" : self.on_btnStart_clicked,
              "on_btnRestart_clicked" : self.on_btnRestart_clicked,
              "on_btnStop_clicked" : self.on_btnStop_clicked,
              "on_mnuAbout_activate" : self.on_mnuAbout_activate,
              "on_mnuManual_activate" : self.on_mnuManual_activate,
              "on_optRL3_activate" : self.on_optRL3_activate,
              "on_optRL4_activate" : self.on_optRL4_activate,
              "on_optRL5_activate" : self.on_optRL5_activate ,
              "on_pmnStart_activate" : self.on_btnStart_clicked,
              "on_pmnStop_activate" : self.on_btnStop_clicked,
              "on_pmnRestart_activate" : self.on_btnRestart_clicked } )

        # main window
        self.winMain = self.xml.get_widget("winMain")
        self.winMain.connect("delete-event", gtk.mainquit)
        self.winMain.connect("hide", gtk.mainquit)

        # menu items
        self.mnuRescan = self.xml.get_widget("mnuRescan")
        self.mnuSave = self.xml.get_widget("mnuSave")
        self.mnuRevert = self.xml.get_widget("mnuRevert")
        self.mnuExit = self.xml.get_widget("mnuExit")
        self.mnuAbout = self.xml.get_widget("mnuAbout")
        self.mnuStart = self.xml.get_widget("mnuStart")
        self.mnuStop = self.xml.get_widget("mnuStop")
        self.mnuRestart = self.xml.get_widget("mnuRestart")
        self.optRL3 = self.xml.get_widget("optRL3")
        self.optRL4 = self.xml.get_widget("optRL4")
        self.optRL5 = self.xml.get_widget("optRL5")
        
        # buttons
        self.btnSave = self.xml.get_widget("btnSave")
        self.btnRevert = self.xml.get_widget("btnRevert")
        self.btnExit = self.xml.get_widget("btnExit")
        self.btnStart = self.xml.get_widget("btnStart")
        self.btnRestart = self.xml.get_widget("btnRestart")
        self.btnStop = self.xml.get_widget("btnStop")

        #toolbars
        self.tbrSave = self.xml.get_widget("tbrSave")
        #self.tbrSave.set_style(GTK.TOOLBAR_ICONS)
        #self.tbrSave.set_button_relief(GTK.RELIEF_NORMAL)
        
        # the textbox
        self.txtDesc = self.xml.get_widget("txtDesc")
        self.txtDesc.set_word_wrap(1)

        self.lblRunlevel = self.xml.get_widget("lblRunlevel")
        self.lblEditing = self.xml.get_widget("lblEditing")
        
        self.editing_runlevel = self.ServiceMethods.get_runlevel()

        self.pmnStart = self.xml.get_widget("pmnStart")
        self.pmnStop = self.xml.get_widget("pmnStop")
        self.pmnRestart = self.xml.get_widget("pmnRestart")
        
        # the CheckCList which will display whether the service
        # is enabled, and the service names        
        
        self.clstServices = CheckList.CheckList()
        self.xml.get_widget("swindow").add(self.clstServices)

        self.clstServices.show()

        self.ServiceMethods = servicemethods.ServiceMethods()

        # initialize this to the runlevel we are running in. It will
        # change when one of the optRL[3-6] are selected
        #self.editing_runlevel = self.ServiceMethods.get_runlevel()

        self.lblRunlevel.set_text (_("Currently Running in Runlevel: ") + self.ServiceMethods.get_runlevel())
        
        # This is a sentry
        self.already_init = 0
        self.populateList()
        self.on_clstServices_select_row(None, 0, 0, None)
        self.clstServices.connect("button_press_event", self.local_button_press_cb)
        self.clstServices.connect("select_row", self.on_clstServices_select_row)
        self.clstServices.connect("unselect_row", self.on_clstServices_unselect_row)
        self.clstServices.set_column_title(0,_("Start at Boot"))
        self.clstServices.set_column_title(1,_("Services"))
        self.clstServices.set_column_justification(0,GTK.JUSTIFY_CENTER)
        self.clstServices.set_column_justification(1,GTK.JUSTIFY_LEFT)
        self.clstServices.set_column_auto_resize(0,1)
        self.clstServices.column_titles_show()

        self.popup_menu = gtk.GtkMenuItem()

        if self.editing_runlevel == "3" or self.editing_runlevel == "2" or self.editing_runlevel == "1":
            self.optRL3.set_active(1)

        elif self.editing_runlevel == "5" :
            self.optRL5.set_active(1)

        elif self.editing_runlevel == "4" :
            self.optRL4.set_active(1)

        self.save_revert_sensitive(0)
        
        self.current_selected_service = self.clstServices.get_text(0,1)
        
        self.winMain.show()



#-----------------------------------------------------------------------------
# Methods assiciated with populating the checklist
#-----------------------------------------------------------------------------
    def populateList(self):
        """Populates clstServices with the service names, whether it is configured, and service descriptions"""
        # sentinal so that we don't have selected rows while we are updating
        self.am_updating = 1

        if self.already_init == 0: 
            # this is passed to ServiceMethods.get_service_list as an
            # argument. We don't want to import gtk there, so we have
            # the actual function here
            def idle_func():
                while gtk.events_pending():
                    gtk.mainiteration()
            self.allservices, self.dict_services = self.ServiceMethods.get_service_list(self.editing_runlevel, idle_func)
            self.dict_services_orig = self.ServiceMethods.dict_services_orig
            self.lblEditing.set_text(_("Editing Runlevel: ") + self.editing_runlevel)
            self.already_init = 1
        
        for servicename in self.allservices:
            self.clstServices.append_row(servicename, int(self.dict_services[servicename][0][int(self.editing_runlevel)]))
                        
        self.am_updating = 0

    def on_clstServices_unselect_row(self,None, None, None, None):
        """does a backward delete in txtDesc"""
        self.txtDesc.backward_delete(self.txtDesc.get_length())
                


    def on_clstServices_select_row(self,None, row, col, None):
        """Populates txtDesc with the service description of the service selected in clstServices"""
        self.txtDesc.backward_delete(self.txtDesc.get_length())
        self.text_in_row = self.clstServices.get_text(row,1)
        current_runlevel = int(self.editing_runlevel)
        self.current_selected = self.clstServices.get_text(row,1)

        #make sure we aren't updating
        if self.am_updating != 1:

            # enables and disables the save and revert buttons/menus
            # if something has been changed from it's orig setting
            for i in range(0,len(self.ServiceMethods.dict_services_orig)-1):
                self.checking_service = self.clstServices.get_text(i,1)
                service_enabled = int("%d" % self.clstServices.row_get_state(i))
                

                if service_enabled == self.ServiceMethods.dict_services_orig[self.checking_service][0][current_runlevel]:
                    self.save_revert_sensitive(0)
                    
                elif service_enabled != self.ServiceMethods.dict_services_orig[self.checking_service][0][current_runlevel]:
                    self.save_revert_sensitive(1)
                    break


        # set the text in txtDesc
        if self.ServiceMethods.dict_services.has_key(self.text_in_row):
            x = self.ServiceMethods.dict_services[self.text_in_row]
            self.txtDesc.insert_defaults(string.strip(x[2]))

        # if an xinetd service is selected, disable these,
        # they'll do nothing
        if self.ServiceMethods.dict_services[self.text_in_row][1] == 1:
            self.action_widgets_sensitive(0)
        else:
            self.action_widgets_sensitive(1)
        return self.text_in_row


            
#----------------------------------------------------------------------------
# Methods pertaining to the "File" menu items
# The "Save Changes", "Revert to Last Save" and "Exit" menu items are handled
# elsewhere by on_btnSave_clicked(), on_btnRevert_clicked(), and
# "Exit" just calls gtk.mainquit()
#----------------------------------------------------------------------------    
    def on_mnuRescan_activate(self,None):
        """clears clstServices, and reruns populateList()"""
        def idle_func():
            while gtk.events_pending():
                gtk.mainiteration()
        # set the cursor while we are busy
        self.winMain.get_window().set_cursor(gtk.cursor_new(GDK.WATCH))

        # hack so that we don't have to self.winMain.set_sensitive(0)
        # which is really ugly. window is never shown, the events are
        # sent there ( to never-never land) until
        # gtk.grab_remove(window) is called        
        window = gtk.GtkWindow()
        gtk.grab_add(window)
        idle_func()
        self.already_init = 0
        self.clstServices.freeze()
        self.clstServices.clear()
        self.populateList()
        gtk.grab_remove(window)
        self.clstServices.thaw()
        self.winMain.get_window().set_cursor(gtk.cursor_new(GDK.LEFT_PTR))
        del window



#----------------------------------------------------------------------------
# Methods pertaining to the "Edit Runlevel" menu items 
#----------------------------------------------------------------------------
    def on_optRL3_activate(self, None):
        """calls  populateList() to repopulate the checklist for runlevel 3"""
        self.editing_runlevel = "3"
        self.lblEditing.set_text(_("Editing Runlevel: ") + self.editing_runlevel)
        self.clstServices.clear()
        self.populateList()



    def on_optRL4_activate(self, None):
        """calls populateList() to repopulate the checklist for runlevel 4"""
        self.editing_runlevel = "4"
        self.lblEditing.set_text(_("Editing Runlevel: ") + self.editing_runlevel)
        self.clstServices.clear()
        self.populateList()



    def on_optRL5_activate(self, None):
        """calls populateList() to repopulate the checklist for runlevel 5"""
        self.editing_runlevel = "5"
        self.lblEditing.set_text(_("Editing Runlevel: ") + self.editing_runlevel)
        self.clstServices.clear()
        self.populateList()



#----------------------------------------------------------------------------
# Methods pertaining to the "Help" menu items, there will be more here
# once I have "real" help :)
#----------------------------------------------------------------------------
    def on_mnuAbout_activate(self,Dummy):
        """Just a silly about dialog"""
        dlg = gnome.ui.GnomeAbout(title = "serviceconf " + VERSION,
                                  copyright= _("Copyright (c) 2001 Red Hat, Inc. All rights reserved."),
                                  list=["Tim Powers <timp@redhat.com>" ,
                                        "Bill Nottingham <notting@redhat.com>"],
                                  comments= _("""This software may be freely redistributed under the terms of the GPL.
                                  
Please report all bugs you find at the Red Hat bug tracking system:
http://bugzilla.redhat.com"""))
        dlg.run_and_close()

#----------------------------------------------------------------------------
# Methods pertaining to the "Help" menu items, there will be more here
# once I have "real" help :)
#----------------------------------------------------------------------------
    def on_mnuManual_activate(self,args):
        gnome.help.goto ("file:///usr/share/doc/serviceconf-" + VERSION + "/index.html")

#----------------------------------------------------------------------------
# Methods pertaining to the toolbar items
# The menu and popup menu items for "Start", "Stop", and "Restart" are
# handled by on_btnStart_clicked(), on_btnStop_clicked(), and 
# on_btnRestart_clicked()
#----------------------------------------------------------------------------
    def on_btnSave_clicked(self, None):
        """Commits the changes made for each service"""
        for i in range(0,len(self.ServiceMethods.dict_services)):
            servicename = self.clstServices.get_text(i,1)
            service_enabled = "%d" % self.clstServices.row_get_state(i)

            self.ServiceMethods.save_changes(servicename, service_enabled, self.editing_runlevel)

        self.save_revert_sensitive(0)


    def on_btnRevert_clicked(self, None):
        """calls populateList() to repopulate the checklist"""
        self.clstServices.freeze()
        self.clstServices.clear()
        self.populateList()
        self.clstServices.thaw()

        self.save_revert_sensitive(0)


    def get_service_action_results(self, servicename, action_type):
        """calls ServiceMethods.service_action_results and displays the results in a dialog box"""

        results = self.ServiceMethods.service_action_results(servicename, action_type, self.editing_runlevel)

        if int(results[0]) != 0:
            dlg = gnome.ui.GnomeErrorDialog(results[1])
            dlg.run_and_close()
        else:
            dlg = gnome.ui.GnomeOkDialog(results[1])
            dlg.run_and_close()

                  

    def on_btnStart_clicked(self,None):
        """calls get_service_action_results to start the selected initscript"""
        self.get_service_action_results(self.text_in_row, "start")



    def on_btnStop_clicked(self,None):
        """calls get_service_action_results to stop the selected initscript"""
        self.get_service_action_results(self.text_in_row, "stop")



    def on_btnRestart_clicked(self,None):
        """calls get_service_action_results to restart the selected initscript"""
        self.get_service_action_results(self.text_in_row, "restart")



#----------------------------------------------------------------------------
# Methods for everything else
#----------------------------------------------------------------------------
    def local_button_press_cb (self, clist, event):
        """checks to see if the third mouse button was clicked. If it was, then bring up the popup menu"""

        info = self.clstServices.get_selection_info(event.x, event.y)
        self.clstServices.select_row(info[0], info[1])
        
        if (event.button == 3) and (info[1] == 1):
            self.clstServices.select_row (info[0], info[1])
            self.popupMenu = self.xml.get_widget ("popup_menu")

            # get rid of the tearoff from the popup menu
            if isinstance (self.popupMenu.children ()[0], gtk.GtkTearoffMenuItem):
                self.popupMenu.remove (self.popupMenu.children ()[0])

            self.pmnBlank = self.xml.get_widget ("pmnBlank")
            # set the label text to be the current selected servicename for pmnBlank
            self.pmnBlank.children ()[1].set_text (self.current_selected)
            self.popupMenu.popup(None, None, None, event.button, event.time)


    def save_revert_sensitive(self, sensitive):
        """sets the save and revert buttons and menus to be sensitive or not, sensitive is a 1 or 0"""
        self.btnSave.set_sensitive(sensitive)
        self.btnRevert.set_sensitive(sensitive)
        self.mnuSave.set_sensitive(sensitive)
        self.mnuRevert.set_sensitive(sensitive)

    def action_widgets_sensitive(self, sensitive):
        """sets the start/stop/restart buttons and menuitems to be sensitive or not"""
        self.btnStart.set_sensitive(sensitive)
        self.btnRestart.set_sensitive(sensitive)
        self.btnStop.set_sensitive(sensitive)
        self.mnuStart.set_sensitive(sensitive)
        self.mnuStop.set_sensitive(sensitive)
        self.mnuRestart.set_sensitive(sensitive)
        self.pmnStart.set_sensitive(sensitive)
        self.pmnStop.set_sensitive(sensitive)
        self.pmnRestart.set_sensitive(sensitive)
                                    
        
def main():
    if os.geteuid() == 0:
        Gui()
        gtk.mainloop()
    else:
        print _("You must run serviceconf as root.")
        sys.exit(-1)


    
if __name__ == "__main__":
    main() 
